<?php

class UloztoSynologyHost
{
	/** @var string */
	private $url;
	/** @var string */
	private $username;
	/** @var string */
	private $password;
	/** @var array */
	private $hostInfo;

	/** @var string */
	private $userAgent;

	/** @var string */
	private $apiBaseUrl = 'https://apis.uloz.to';
	/** @var bool */
	private $apiVerifySSL = true;
	/** @var string */
	private $apiKey = 'g{Khbn8!&24CZcVw[)H-';
	/** @var string */
	private $apiSessionToken = '';

	/**
	 * Implements Synology Download Station host module pseudo interface
	 * @param string $url
	 * @param string $username
	 * @param string $password
	 * @param array $hostInfo
	 */
	public function __construct($url, $username, $password, $hostInfo)
	{
		$this->url = preg_replace('%^http://%i', 'https://', $url);
		$this->username = $username;
		$this->password = $password;
		$this->hostInfo = $hostInfo;

		$this->userAgent = "Synology module {$hostInfo['version']} on " . DOWNLOAD_STATION_USER_AGENT;
	}


	/**
	 * @param string $apiBaseUrl
	 * @param string $apiKey
	 * @param bool $apiVerifySSL
	 */
	public function setup($apiBaseUrl, $apiKey, $apiVerifySSL)
	{
		$this->apiBaseUrl = $apiBaseUrl;
		$this->apiKey = $apiKey;
		$this->apiVerifySSL = $apiVerifySSL;
	}


	/**
	 * Implements Synology Download Station host module pseudo interface
	 * @param $clearCookie
	 * @return int
	 */
	public function Verify($clearCookie)
	{
		try {
			$this->apiSessionToken = $this->getSessionToken();
			if ($this->apiSessionToken !== '') {
				return USER_IS_PREMIUM;
			}
		} catch (Exception $e) {
			// do nothing...
		}
		return LOGIN_FAIL;
	}


	/**
	 * Implements Synology Download Station host module pseudo interface
	 * @return array|object|false
	 */
	public function GetDownloadInfo()
	{
		$path = parse_url($this->url, PHP_URL_PATH);
		if (!preg_match('%^/(((live/)?!)|(file/))(?<slug>[a-z0-9]+)/%i', $path, $matches)) {
			return [DOWNLOAD_ERROR => ERR_NOT_SUPPORT_TYPE];
		}

		try {
			if ($this->Verify(false) === LOGIN_FAIL) {
				return [DOWNLOAD_ERROR => ERR_REQUIRED_ACCOUNT];
			} else {
				return $this->getDownloadUrl($matches['slug']);
			}
		} catch (Exception $e) {
			return [DOWNLOAD_ERROR => ERR_UNKNOWN];
		}
	}


	/**
	 * @return string
	 */
	private function getSessionToken()
	{
		$apiResponse = $this->callApi('PUT', '/session/v1', [
			'login' => $this->username,
			'password' => $this->password,
		]);
		if ($apiResponse['code'] === 201) {
			return (string)$apiResponse['json']['token_id'];
		}
		return '';
	}


	/**
	 * @param string $fileSlug
	 * @return array
	 */
	private function getDownloadUrl($fileSlug)
	{
		$apiResponse = $this->callApi('GET', '/v5/file/public/!' . urlencode($fileSlug));
		if ($apiResponse['code'] !== 200 && $this->apiSessionToken) {
			$apiResponse = $this->callApi('GET', '/v5/file/private/!' . urlencode($fileSlug), null,
				['X-User-Token: ' . $this->apiSessionToken]);
			if ($apiResponse['code'] !== 200) {
				return [DOWNLOAD_ERROR => ERR_FILE_NO_EXIST];
			}
		}

		$successResponse = [
			DOWNLOAD_ISPARALLELDOWNLOAD => true,
			DOWNLOAD_FILENAME => $apiResponse['json']['name'],
		];

		$curl = curl_init($this->url);

		curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
		curl_setopt($curl, CURLOPT_USERPWD, "{$this->username}:{$this->password}");
		curl_setopt($curl, CURLOPT_USERAGENT, $this->userAgent);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, $this->apiVerifySSL);
		curl_setopt($curl, CURLOPT_FOLLOWLOCATION, false);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

		curl_exec($curl);
		$status = (int)curl_getinfo($curl, CURLINFO_RESPONSE_CODE);
		$location = (string)curl_getinfo($curl, CURLINFO_REDIRECT_URL);
		curl_close($curl);

		if ($status >= 300 && $status < 400 && strpos($location, '/Ps;Hs;') !== false) {
			$successResponse[DOWNLOAD_URL] = $location;
			return $successResponse;
		}

		return [DOWNLOAD_ERROR => ERR_FILE_NO_EXIST];
	}


	/**
	 * @param string $method
	 * @param string $endpoint
	 * @param array|null $body
	 * @param array $headers
	 * @return array
	 */
	private function callApi($method, $endpoint, $body = null, $headers = [])
	{
		$body = $body ? json_encode($body) : '';
		$curl = curl_init($this->apiBaseUrl . $endpoint);

		curl_setopt($curl, CURLOPT_USERAGENT, $this->userAgent);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, $this->apiVerifySSL);
		curl_setopt($curl, CURLOPT_HTTPHEADER, array_merge([
			'Accept: application/json',
			'Content-Type: application/json',
			'Content-Length: ' . strlen($body),
			'X-Auth-Token: ' . $this->apiKey,
		], $headers));
		if ($body) {
			curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
		}

		$response = curl_exec($curl);
		$error = curl_error($curl);
		$responseCode = curl_getinfo($curl, CURLINFO_RESPONSE_CODE);
		curl_close($curl);

		if (!$response) {
			throw new RuntimeException('Api call failed: ' . $error);
		}

		$responseJson = json_decode($response, true);
		if (!$responseJson) {
			throw new RuntimeException("Unexpected response: $response");
		}

		return ['code' => $responseCode, 'json' => $responseJson];
	}
}
